@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): moz_bug_r_a4
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,21 +35,40 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-476871-01.js';
+var gTestfile = 'regress-367119-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 476871;
-var summary = 'Do not assert: *(JSObject**)slot == NULL';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 367119;
+var summary = 'memory corruption in script_exec';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
 
-jit(true);
+function test()
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
+ 
+  if (typeof Script == 'undefined')
+  {
+    print('Test skipped. Script or toSource not defined');
+  }
+  else
+  {
+    var s = new Script("");
+    var o = {
+      valueOf : function() {
+        s.compile("");
+        Array(11).join(Array(11).join(Array(101).join("aaaaa")));
+        return {};
+      }
+    };
+    s.exec(o);
+  }
+  reportCompare(expect, actual, summary);
 
-let ([] = false) { (this.watch("x", /a/g)); };
-(function () { (eval("(function(){for each (x in [1, 2, 2]);});"))(); })();
-
-jit(false);
-
-reportCompare(expect, actual, summary);
+  exitFunc ('test');
+}
